@@ -211,7 +211,15 @@ def generate_alleles_and_xx_codes_and_who(db_connection: sqlite3.Connection, img
                                 ('allele_1d', 'allele_list'))
         xx_codes = {k: v.split('/') for k, v in xx_codes.items()}
 
-        return valid_alleles, who_alleles, xx_codes, who_group
+        shortnulls = db.load_dict(db_connection, 'shortnulls', 
+                                ('shortnull', 'allele_list'))
+        shortnulls = {k: v.split('/') for k, v in shortnulls.items()}
+
+        exp_alleles = db.load_dict(db_connection, 'exp_alleles', 
+                                ('exp_allele', 'allele_list'))
+        exp_alleles = {k: v.split('/') for k, v in exp_alleles.items()}
+
+        return valid_alleles, who_alleles, xx_codes, who_group, shortnulls, exp_alleles
 
     # Create a Pandas DataFrame from the mac_code list file
     # Skip the header (first 6 lines) and use only the Allele column
@@ -229,6 +237,12 @@ def generate_alleles_and_xx_codes_and_who(db_connection: sqlite3.Connection, img
     exp_alleles = allele_df[allele_df['Allele'].apply(
         lambda a: a[-1] in expression_chars and number_of_fields(a) > 2)]
     exp_alleles = exp_alleles.groupby('2d').apply(expression_reduce).dropna()
+
+    #flat_exp_alleles = {k: '/'.join(sorted(v, key=functools.cmp_to_key(smart_sort_comparator)))
+    #                 for k, v in exp_alleles.items()}
+    db.save_dict(db_connection, 'exp_alleles', exp_alleles,
+                 ('exp_allele', 'allele_list'))
+
     # Create valid set of alleles:
     # All full length alleles
     # All 3rd and 2nd field versions of longer alleles
@@ -304,7 +318,35 @@ def generate_alleles_and_xx_codes_and_who(db_connection: sqlite3.Connection, img
     db.save_dict(db_connection, 'who_group', flat_who_group,
                  columns=('who', 'allele_list'))
 
-    return valid_alleles, who_alleles, xx_codes, who_group
+    # shortnulls
+    # scan WHO alleles for those with expression characters and make shortnull mappings 
+    # DRB4*01:03N | DRB4*01:03:01:02N/DRB4*01:03:01:13N 
+    # DRB5*01:08N | DRB5*01:08:01N/DRB5*01:08:02N 
+    shortnulls = dict()
+    for who in who_group:
+        # e.g. DRB4*01:03
+        expression_alleles = dict()
+        if who[-1] not in expression_chars and who[-1] not in ['G', 'P'] and ":" in who:
+            for an_allele in who_group[who]:
+                # if an allele in a who_group has an expression character but the group allele doesnt, 
+                # add it to shortnulls
+                last_char = an_allele[-1]
+                if last_char in expression_chars:
+                    # e.g. DRB4*01:03:01:02N
+                    a_shortnull = who + last_char
+                    if a_shortnull not in expression_alleles:
+                        expression_alleles[a_shortnull] = []
+                    expression_alleles[a_shortnull].append(an_allele)
+            # only create a shortnull if there is one expression character in this who_group
+            # there is nothing to be done for who_groups that have both Q and L for example
+            for a_shortnull in expression_alleles:
+                # e.g. DRB4*01:03N 
+                shortnulls[a_shortnull] = "/".join(expression_alleles[a_shortnull])
+
+    db.save_dict(db_connection, 'shortnulls', shortnulls, ('shortnull', 'allele_list'))
+    shortnulls = {k: v.split('/') for k, v in shortnulls.items()}
+
+    return valid_alleles, who_alleles, xx_codes, who_group, shortnulls, exp_alleles
 
 
 def generate_mac_codes(db_connection: sqlite3.Connection, refresh_mac: bool):
@@ -358,7 +400,8 @@ def generate_mac_codes(db_connection: sqlite3.Connection, refresh_mac: bool):
         # Load the MAC file to a DataFrame
         mac_url = 'https://hml.nmdp.org/mac/files/numer.v3.zip'
         df_mac = pd.read_csv(mac_url, sep='\t', compression='zip',
-                             skiprows=3, names=['Code', 'Alleles'])
+                             skiprows=3, names=['Code', 'Alleles'],
+                             keep_default_na=False)
         # Create a dict from code to alleles
         mac = df_mac.set_index("Code")["Alleles"].to_dict()
         # Save the mac dict to db
